<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Service\Sm00001100LancamentoPropostaService as PropostaService;
use Teknisa\SM\Service\Sm00004000ContratoService;
use Teknisa\SM\Util\Utilities;

class Sm00001100LancamentoPropostaController extends SMCrud {
    protected $dataSourceName       = "fct_orcamento_widget";
    protected $dataSourceNameToSave = 'fct_orcamento';
    protected $newCode;
    protected $propostaService;
    protected $contratoService;
    
    public function __construct($crudService, $environment, NewCode $newCode, PropostaService $propostaService, Sm00004000ContratoService $contratoService) {
        parent::__construct($crudService, $environment);
        $this->newCode             = $newCode;
        $this->propostaService     = $propostaService;
        $this->contratoService     = $contratoService;
    }
    /**
    public function getServicosPropostaInserindo(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            $nrmodelo        = $row["NRORCAMENTOMODELO"];
            $nrorg           = $this->environment->getNrOrg();
            $dataSet = $this->propostaService->getServicosPropostaInserindo($nrtipoorcamento, $nrorg, $nrmodelo);
            $response->addDataSet(new DataSet("composicao_elemento_proposta_alimenticia", $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }**/
    /**
     * Salva a Proposta e seus Escopos, chamando as procedures CRIA_MNEMONICO e FCT_GERA_DADOS_PROPOSTA.
     * 
     * @param Request $request
     * @param Response $response
     */
    public function save(Request $request, Response $response) {
        try {
            $request->setParameter("dbname", $this->dataSourceName);
            $nrorg     = $this->environment->getNrOrg();
            $qtdLinhas = count($request->getDataSet()->getRows());
            $row       = $request->getDataSet()->getRows()[$qtdLinhas-1];
            $escopos   = $row["NRTIPOORCAMENTO"];
            $modelo    = $row["NRORCAMENTOMODELO"];

            $row["NRTIPOORCAMENTO"] = null;
            if ($row["__is_new"]) {
                $nrseqorcamento = $this->newCode->getCodeByTableName("NRSEQORCAMENTO", $nrorg, 1, 10, true);
                $row["NRSEQORCAMENTO"] = $nrseqorcamento;
                parent::save($request, $response);
                
                $mnemonico = $this->propostaService->executarProcedureCriaMnemonico($nrorg, "O", $nrseqorcamento);
                $this->propostaService->executarProcedureFctGeraDadosProposta($nrseqorcamento, $nrorg);
                $this->propostaService->anexaDocumento($nrseqorcamento, $nrorg);
                
                $columnsToReturn = $this->crudService->getColumnsToReturn();
                $columnsToReturn[0]["NRSEQORCAMENTO"] = $nrseqorcamento;
                $columnsToReturn[0]["NRORCAMENTO"]    = $mnemonico;
                $response->addDataSet(new DataSet("columnsToReturn", $columnsToReturn));
            } else {
                parent::save($request, $response);
                $nrseqorcamento = $row["NRSEQORCAMENTO"];
                $this->propostaService->excluirEscopos($nrseqorcamento, $nrorg);
            }
            
            if (!empty($escopos)) {
                if (!is_array($escopos))
                    $escopos = array($escopos);
                $this->salvarEscopos($escopos, $modelo, $nrseqorcamento, $nrorg, $request->getUserId());
            }
            $response->addDataSet(new DataSet("NRSEQORCAMENTO", array($nrseqorcamento)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Salva os Escopos da Proposta.
     * @param integer[] $escopos
     * @param integer $modelo
     * @param integer $nrseqorcamento
     * @param integer $nrorg
     * @param mixed[] $userId
     */
    private function salvarEscopos($escopos, $modelo, $nrseqorcamento, $nrorg, $userId) {
        try {
            $dataset  = array();
            $operador = $this->environment->getCdOperador();
            foreach ($escopos as $escopo) {
                $nrrelacescopoproposta = $this->newCode->getCodeByTableName("FCT_RELACESCOPOPROPOSTA", $nrorg, 1, 10, true);
                $documento = null;
                $cdfilial  = null;

                $documentoAux = $this->propostaService->buscaModeloDocumento($modelo, $escopo, $nrorg); 
                if ($documentoAux) {
                    $documento = $documentoAux[0]["NRSEQMODDOCFCT"];
                    $cdfilial = $documentoAux[0]["CDFILIAL"];
                }
                
                $FilialHier = $this->propostaService->buscaFilialHierarquia($operador, $nrorg);
                if($FilialHier)
                    $cdfilial = $FilialHier[0]["CDFILIAL"];    

                $dataset[] = array(
                    "__is_new"              => true,
                    "NRRELACESCOPOPROPOSTA" => $nrrelacescopoproposta,
                    "NRSEQORCAMENTO"        => $nrseqorcamento,
                    "NRTIPOORCAMENTO"       => $escopo,
                    "NRSEQMODDOCFCT"        => $documento,
                    "CDFILIAL"              => $cdfilial,
                    "NRORG"                 => $nrorg,
                    "IDATIVO"               => "S"
                );
            }
            
            $requestRelac = new Request\DataSet(new DataSet("", $dataset), "product#/fct_relacescopoproposta/save", "POST", $userId);
            $response     = new Response();
            $this->dataSourceNameToSave = "fct_relacescopoproposta";
            $requestRelac->setParameter('dbname', 'fct_relacescopoproposta');
            parent::save($requestRelac, $response);
            $this->dataSourceNameToSave = "fct_orcamento";
        } catch (\Exception $e) {
            throw $e;
        }
    }
    /**
     * Manda inserir o serviço, mão de obra, despesa, investimento, taxa, imposto
     * ligados a proposta
    **/
    public function inserirTudoProposta(Request\Row $request, Response $response){
        try{
            $row             = $request->getRow();
            $servicos        = $row["SERVICOS"];
            $mObras          = $row["MOBRAS"];
            $despesas        = $row["DESPESAS"];
            $investimentos   = $row["INVESTIMENTOS"];
            $taxas           = $row["TAXAS"];
            $nrseqorcamento  = $row["NRSEQORCAMENTO"];
            $itens           = $row["ITENS"];
            $custos          = $row["CUSTOS"];
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            $impostos        = $row["IMPOSTO"];
            $servicoRelac    = $row["SERVICORELAC"];
            $nrorg           = $this->environment->getNrOrg();
            $cdoperador      = $this->environment->getCdOperador();
            
            $this->propostaService->inserirTudoProposta($itens, $custos, $servicos, $mObras, $despesas, $investimentos, $taxas, $nrseqorcamento, $nrorg, $cdoperador, $nrtipoorcamento, $impostos, $servicoRelac);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Retorna as informações do serviço para a widget de custo do serviço
    **/
    public function getComposicoesServicos(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            $nrcomposicao    = $row["NRCOMPSERVICO"];
            $nrorg           = $this->environment->getNrOrg();
            
            $dataset = array();
            if(count($nrtipoorcamento) > 0 && count($nrcomposicao) > 0)
                $dataset = $this->propostaService->getComposicoesServicos($nrtipoorcamento, $nrorg, $nrcomposicao);
                
            $response->addDataSet(new DataSet("fct_custocomposicao", $dataset));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Retorna as infomações do resultado de propostas já inseridas 
    **/
    public function getInfoResultado(Request\Row $request, Response $response){
        try{
            $row   = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $dataset = $this->propostaService->getInfoResultado($nrseqorcamento, $nrorg);
            $response->addDataSet(new DataSet("getInfoResultado", $dataset));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Atualiza o valor da proposta com a somatória do valor de seus itens.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function atualizarValorProposta(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $nrorg = $this->environment->getNrOrg();
            $valor = $this->propostaService->atualizarValorProposta($nrseqorcamento, $nrorg);
            $response->addDataSet(new DataSet("FCT_ORCAMENTO", array("VRORCAMENTO" => $valor)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Manda atualizar o valor do item da proposta 
     * apos a alteração do valor de algum componente do serviço
    **/
    public function atualizarValorItem(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getNrOrg();
            $this->propostaService->atualizarValorItem($nrseqorcamento, $nrorg, $cdoperador);
            
            $vrorcamento =  $this->propostaService->atualizarValorProposta($nrseqorcamento, $nrorg);
            $response->addDataSet(new DataSet("fct_orcamentoaliment_widget", array("VRORCAMENTO" => $vrorcamento)));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Aceita a Proposta.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function aceitarProposta(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqorcamento    = $row["NRSEQORCAMENTO"];
            $nrorg             = $row["NRORG"];
            $utilizaComposicao = $row["IDUTILIZACOMP"];
            $nrparcnegocio     = $row["NRPARCNEGOCIO"];
            $periodoExtendido     = isset($row["PERIODOEXTENDIDO"]) ? true : false;
            $dtFinVigContr = false;
            if($periodoExtendido == true){
                $dtFinVigContr     = $row["DTFINVIGCONTR"];
            }
            $cdoperador = $this->environment->getCdOperador();
            $this->propostaService->aceitarProposta($nrseqorcamento, $nrorg, $utilizaComposicao, $nrparcnegocio, $cdoperador, $periodoExtendido, $dtFinVigContr);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Cancela a Proposta.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function cancelarProposta(Request\Row $request, Response $response) {
        try {
            $userId = $request->getUserId();
            $row = $request->getRow();
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $nrorg          = $row["NRORG"];
            $propostas = array($nrseqorcamento);
            $contratos = $this->contratoService->getCodigoContratosDasPropostas($propostas, $nrorg);
            if (is_array($contratos)) 
                $this->contratoService->reprovarContratos($contratos, $nrorg);

            $this->propostaService->atualizarSituacaoPropostas("CANCELADO", $propostas, $nrorg);
            //$this->itensService->atualizarAgendamentoPendente($nrseqorcamento, $nrorg);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Copia ou versiona a Proposta.
     * 
     * @param Request\Row $request
     * @param Response $response
     **/
    public function copiarVersionarProposta(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $tipo           = $row["TIPO"];
            $nrorg          = $this->environment->getNrOrg();
            $cdoperador     = $this->environment->getCdOperador();
            $propostaGerada = null;
            if($tipo == 'V' && $row["IDSITUACAOORCAMENTO"] == "ACEITO"){
                $propostas = array($nrseqorcamento);
                $contratos = $this->contratoService->getCodigoContratosDasPropostas($propostas, $nrorg);
                if (is_array($contratos)) 
                    $this->contratoService->reprovarContratos($contratos, $nrorg);
            }
            
            $propostaGerada = $this->propostaService->executarProcedureFctCopiaVersPropostaAliment($nrseqorcamento, $tipo, $cdoperador, $nrorg);
            $this->propostaService->anexarDocumentosPropostaCopiada($propostaGerada, $nrorg, $cdoperador);
            //if($tipo == 'V')
                //$this->itensService->excluiItemAgendPendente($nrseqorcamento, null, $nrorg);
            $response->addDataSet(new DataSet("NRSEORCGERADO", array($propostaGerada)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Retorna o código da ultima pk inserida na tabela de fct_relacformacaoprop 
    **/
    public function getCodeRelacFormacaoProp(Request\Row $request, Response $response){
        try {
            $nrorg = $this->environment->getNrOrg();
            $nrrelacForm = $this->propostaService->getCodeRelacFormacaoProp();
            $response->addDataSet(new DataSet("fct_relacformacaoprop", array("NRULTRELACFORM" => $nrrelacForm)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Alterar os valores no orcamento de acordo com o novo valor informado 
     * na alteração do valor do servico de uma proposta ja inserida
    **/
    public function alterarValorServicoTotalItem(Request\Row $request, Response $response){
        try{
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrorg();
            $cdoperador = $this->environment->getCdOperador();
            $nrseqorcamento = $row["NRSEQORCAMENTO"];
            $nrcompraiz     = $row["NRCOMPSERVICO"];
            $qtservico      = $row["QTITEMORCAMENTO"];
            $vrunitServico  = $row["NEWVRSERVICO"];
            $totalOrcamento = $this->propostaService->atualizarItensAlteracaoResultado($nrseqorcamento, $nrorg, $cdoperador, $nrcompraiz, $qtservico, $vrunitServico);

            $dataset = $this->propostaService->getInfoResultado($nrseqorcamento, $nrorg);
            $dataset = array_merge($dataset, array("VRORCAMENTO" => $totalOrcamento));
            $response->addDataSet(new DataSet("getInfoResultado", $dataset));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Salva as alterações dos itens de formação da proposta
    **/
    public function utilizarItensFormacaoProp(Request $request, Response $response){
        try{
            $rows   = $request->getDataSet()->getRows();
            $userId = $request->getUserId();  
            $this->dataSourceNameToSave = "fct_calcformprecserv_itpro_widget";
            foreach($rows as $row){
                $requestRelac = new Request\DataSet(new DataSet("", [$row]), "product#/fct_calcformprecserv_itpro_widget/save", "POST", $userId);
                $requestRelac->setParameter('dbname', 'fct_calcformprecserv_itpro_widget');
                parent::save($requestRelac, $response);
            }
            $this->dataSourceNameToSave = "fct_orcamento";
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}